#include <Eigen/Core>
#include <vector>
#include <iostream>
#include "writer.hpp"

/// Uses the Heun's method to compute u from time 0 to time T 
/// for the ODE $u'=e^{-t}-u$
///
/// @param[out] u at the end of the call (solution at all time steps)
/// @param[out] time contains the time levels
/// @param[in] u0 the initial data
/// @param[in] dt the step size
/// @param[in] T the final time up to which to compute the solution.
///

void Heun(std::vector<double> & u, std::vector<double> & time,
	const double & u0, double dt, double T) {
  const unsigned int nsteps = round(T/dt);
  u.resize(nsteps+1);
  time.resize(nsteps+1);
  
  time[0] = 0.;
  u[0] = u0;
  
  for(unsigned i=0; i<nsteps; i++)
    {
      time[i+1] = (i+1)*dt;
      double k1 = std::exp(-1.*time[i])-u[i];
      double k2 = std::exp(-1.*time[i+1])-u[i]-dt*k1;
      u[i+1] = u[i] + dt*0.5*(k1+k2);     
    }
}

int main() {
	
	double T = 10.0;
	double dt = 1.5;
	const double u0 = 0.;
	std::vector<double> time;
	std::vector<double> u;
	Heun(u,time,u0,dt,T);
	writeToFile("solution.txt", u);
	writeToFile("time.txt",time);
	return 0;
}
