#include <writer.hpp>
#include <Eigen/Core>
#include <Eigen/LU>
#include <vector>
#include <iostream>

///
/// Compute N timesteps of Forward-Euler
/// @param[in/out] should be a vector of size 3, where each
///                component is a vector of size N+1. u[i][0]
///                should have the initial value stored before
///                calling the funtion
///
/// @param[out] time should be of length N+1
///
/// @param[in] T the final time
/// @param[in] N the number of timesteps
/// @param[in] k1 the constant k1
/// @param[in] k2 the constant k2
///
void forwardEuler(std::vector<std::vector<double> >& u, std::vector<double>& time,
		  double T, int N, double k1, double k2) {
    const double dt = T / N;

    Eigen::Matrix3d A;
    A << -k1, 0., 0.
	, k1, -k2, 0.
	, 0., k2, 0.;

    for (int i = 1; i < N + 1; ++i) {
	Eigen::Vector3d uPrevious(u[0][i - 1], u[1][i - 1], u[2][i - 1]);
	Eigen::Vector3d uNext = uPrevious + dt * (A * uPrevious);

	u[0][i] = uNext[0];
	u[1][i] = uNext[1];
	u[2][i] = uNext[2];

	time[i] = time[i - 1] + dt;
    }
}

int main(int argc, char** argv) {
    double k1 = 1e4;
    double k2 = 1;

    double T = 1;
    int N = 5e3;
    if (argc > 1)  {
	// Read N from command line
	// We use atof because we want to allow things like 1e7
	N = int(atof(argv[1]));
    }
    std::vector<std::vector<double> > u(3);
    std::vector<double> time(N + 1, 0);
    u[0].resize(N + 1, 0);
    u[1].resize(N + 1, 0);
    u[2].resize(N + 1, 0);

    u[0][0] = 1;
    u[1][0] = 0;
    u[2][0] = 0;


    forwardEuler(u, time, T, N, k1, k2);

    writeToFile("u0.txt", u[0]);
    writeToFile("u1.txt", u[1]);
    writeToFile("u2.txt", u[2]);
    writeToFile("time.txt", time);
}




